//*****************************************************************************
//   +--+
//   | ++----+
//   +-++    |
//     |     |
//   +-+--+  |
//   | +--+--+
//   +----+    Copyright (c) 2009 Code Red Technologies Ltd.
//
// main_bootloader.c
//        - flash-based USB secondary bootloader for RDB1768 development board
//          fitted with NXP LPC1768 MCU. Sits on top of LPCUSB mass storage
//          example (plus new driver level to access LPC1768's flash).
//			Principles of bootloader based on NXP AN10866 example.
//
// Software License Agreement
//
// The software is owned by Code Red Technologies and/or its suppliers, and is
// protected under applicable copyright laws.  All rights are reserved.  Any
// use in violation of the foregoing restrictions may subject the user to criminal
// sanctions under applicable laws, as well as to civil liability for the breach
// of the terms and conditions of this license.
//
// THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
// OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
// USE OF THIS SOFTWARE FOR COMMERCIAL DEVELOPMENT AND/OR EDUCATION IS SUBJECT
// TO A CURRENT END USER LICENSE AGREEMENT (COMMERCIAL OR EDUCATIONAL) WITH
// CODE RED TECHNOLOGIES LTD.
//
//*****************************************************************************
#include <cr_section_macros.h>
#include <NXP/crp.h>

#include "lpc17xx_ssp.h"
#include "lpc17xx_gpio.h"
#include "lpc17xx_pinsel.h"

#include <stdint.h>
#include "sbl_iap.h"
#include "sbl_config.h"

#include "ff.h"
#include "diskio.h"
#include "LPC17xx.h"
#include "string.h"

Bool  user_flash_erased;
static FATFS Fatfs[1];

DSTATUS stat;
DWORD p2;
WORD w1;
BYTE b1;
UINT br;

#define  LED_PIN		24			   /* Port 2 pin */

static int fatfs_init(void);
int BlockDevWrite(uint32_t dwAddress, uint8_t * pbBuf);

const char *FIRMWARE_PATH = "/firmware";
const char *FIRMWARE_LOG = "/firmware/log.txt";

// Variable to store CRP value in. Will be placed automatically
// by the linker when "Enable Code Read Protect" selected.
// See crp.h header for more information
__CRP const unsigned int CRP_WORD = CRP_NO_CRP ;

DWORD get_fattime (void){

	DWORD time = 0;
	return time;

}
static void init_ssp(void)
{
	SSP_CFG_Type SSP_ConfigStruct;
	PINSEL_CFG_Type PinCfg;


    /*
     * Initialize SPI0 pin connect
     * P0.15 - SCK;
     * P0.17 - MISO
     * P0.18 - MOSI
     * P0.16 - SSEL - used as GPIO
     */

    PinCfg.Funcnum = 2;
    PinCfg.OpenDrain = 0;
    PinCfg.Pinmode = 0;
    PinCfg.Portnum = 0;
    PinCfg.Pinnum = 15;
    PINSEL_ConfigPin(&PinCfg);
    PinCfg.Pinnum = 17;
    PINSEL_ConfigPin(&PinCfg);
    PinCfg.Pinnum = 18;
    PINSEL_ConfigPin(&PinCfg);
    PinCfg.Funcnum = 0;
    PinCfg.Portnum = 0;
    PinCfg.Pinnum = 16;
    PINSEL_ConfigPin(&PinCfg);

    SSP_ConfigStructInit(&SSP_ConfigStruct);   // ClockRate = 100000;
    SSP_ConfigStruct.ClockRate = 2000000;
    SSP_Init(LPC_SSP0, &SSP_ConfigStruct);

     // Enable SSP peripheral
    SSP_Cmd(LPC_SSP0, ENABLE);

}

/*****************************************************************************
 * enter_usb_isp() is the routine called if the bootloader determines that the
 * USB interface needs starting to allow the user to upload a new application
 *  into the LPC1768's flash memory
 *****************************************************************************/
void enter_sdcard_isp(void)
{

	uint32_t addr = 0;
	FIL_t f;
	FIL_t f_log;
	DIR_t dir;
	FILINFO fno;
	char *fn;
	volatile BYTE res;
	UINT br;         /* File read/write count */
	UINT wr_err;
	uint8_t firmWrite = FALSE;
	uint8_t buf[FLASH_BUF_SIZE];
	char firmPath[20];

	user_flash_erased = FALSE;

	/* Init SSP for SDCARD */
	init_ssp();
	fatfs_init();


	res = f_opendir(&dir,FIRMWARE_PATH);

	if(res == FR_OK){

		for (;;) {

			res = f_readdir(&dir, &fno);                   /* Read a directory item */
			if (res != FR_OK || fno.fname[0] == 0) break;  /* Break on error or end of dir */
			if (fno.fname[0] == '.') continue;             /* Ignore dot entry */
			fn = fno.fname;

			if ( !(fno.fattrib & AM_DIR)) {
				if(strstr(fn,".bin") || strstr(fn,".BIN")){
					strcpy(firmPath,FIRMWARE_PATH);
					strcat(firmPath,"/");
					strcat(firmPath,fn);
					res = f_open(&f,firmPath,FA_OPEN_EXISTING | FA_READ);
					firmWrite = TRUE;
					break;
				}
			}

		}

	}else
		return;

	if(firmWrite == TRUE ){

		DWORD s = f_size(&f);
		if(s <= 0)
			return;

		erase_user_flash();
		do{
			res = f_read(&f, buf,FLASH_BUF_SIZE, &br);     /* Read a chunk of src file */
			if(res == FR_OK){
				wr_err = BlockDevWrite(addr,buf);
				addr++;
			}
		}while(res == FR_OK && br != 0 && !wr_err);

		f_close(&f);


		res = f_open(&f_log,FIRMWARE_LOG,FA_OPEN_ALWAYS | FA_WRITE);		/* write to the log */
		res = f_lseek(&f_log, f_size(&f_log));								/* move to end of file */

		/* We are done writting to flash now lets delete the file so we dont use it again */
		if(!wr_err && user_code_present()){
			f_printf(&f_log, "-- Firmware Update Completed: %s --\r\n", firmPath);
			f_close(&f_log);
			res = f_open(&f,firmPath,FA_OPEN_EXISTING);
			f_lseek(&f, 0);
			f_truncate(&f);
			f_sync(&f);
			f_unlink(firmPath);
		}else{
			f_printf(&f_log, "-- Firmware Write Error: %s --\r\n", firmPath);
			f_close(&f_log);
		}




	}

	f_mount(0, NULL);

}


void SysTick_Handler(void) {
	disk_timerproc();
  return;                                    /* increment timeTicks counter */
}
/*********************************
 * Main entry point for bootloader
 *********************************/

int main (void)
{

	uint32_t t;

	SysTick_Config (SystemCoreClock / 1000);

	LPC_GPIO1->FIODIR |= (1UL << LED_PIN);

	// User code not present or isp entry requested
	enter_sdcard_isp();

	// Check to see if there is a user application in the LPC1768's flash memory.
	if( user_code_present() )
	{

		execute_user_code();
		// There is an application, but need to check if user is pressing the button
		// to indicate they want to upload a new application.
		//check_isp_entry_pin();
	}

	// Note - should never actually return from enter_sdcard_isp ().
	while (1){
		for(t=9000000; t > 0; t--);
		   if (LPC_GPIO1->FIOPIN & (1<<LED_PIN))
			 LPC_GPIO1->FIOCLR |= (1<<LED_PIN);
		   else
			 LPC_GPIO1->FIOSET |= (1<<LED_PIN);

	}
	return 0;
}

static int fatfs_init(void)
{

		volatile BYTE res;
		DIR_t dir;
		volatile uint32_t t;

		for(t=0xfffff; t > 0; t--);
		for(t=0xfffff; t > 0; t--);

		stat = disk_initialize(0);
	    if (stat & STA_NOINIT)
	    	  return -1;

	    if (stat & STA_NODISK)
	    	return -1;

	    if (stat != 0)
	        return -1;

	    res = f_mount(0, &Fatfs[0]);
	    if (res != FR_OK)
	        return -1;

	    res = f_opendir(&dir, "/");
	    if (res)
	    	return -1;

	    return 0;

}



